<?php

namespace SGalinski\SgJobs\SignalSlot;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManager;
use TYPO3\CMS\Extbase\Mvc\Web\Routing\UriBuilder;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;
use TYPO3\CMS\Frontend\Page\PageGenerator;
use TYPO3\CMS\Frontend\Utility\EidUtility;

/**
 * Signal functions regarding sg_seo sitemap generation
 */
class SitemapSignalSlot {

	const PLUGIN_NAME = 'sgjobs_jobapplication';

	/**
	 * @var UriBuilder
	 */
	protected $uriBuilder;

	/**
	 * before the sitemap is generated
	 *
	 * @param array $pageList
	 * @throws \TYPO3\CMS\Core\Error\Http\ServiceUnavailableException
	 */
	public function beforeSitemapGeneration(array &$pageList) {
		$objectManager = GeneralUtility::makeInstance(ObjectManager::class);
		$request = $objectManager->get(\TYPO3\CMS\Extbase\Mvc\Web\Request::class);
		$request->setRequestUri(GeneralUtility::getIndpEnv('TYPO3_REQUEST_URL'));
		$request->setBaseUri(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'));

		/** @var ObjectManager $objectManager */
		$objectManager = GeneralUtility::makeInstance(ObjectManager::class);
		/** @var ContentObjectRenderer $contentObjectRenderer */
		$contentObjectRenderer = $objectManager->get(ContentObjectRenderer::class);

		/** @var ConfigurationManager $configurationManager */
		$configurationManager = $objectManager->get(ConfigurationManager::class);
		$configurationManager->setContentObject($contentObjectRenderer);
		$this->uriBuilder = $objectManager->get(UriBuilder::class);
		$this->uriBuilder->injectConfigurationManager($configurationManager);

		$this->initTSFE();

		/** @var $databaseConnection DatabaseConnection */
		$databaseConnection = $GLOBALS['TYPO3_DB'];

		// find sites where job detail plugin is added
		$rows = $databaseConnection->exec_SELECTgetRows(
			'pid, pages',
			'tt_content',
			'CType = "list" and list_type = "' . self::PLUGIN_NAME . '" and hidden = 0 and deleted = 0'
		);

		foreach ($rows as $row) {
			$jobs = $this->getJobsByPid($row['pages']);

			foreach ($jobs as $job) {
				$url = $this->uriBuilder->reset()->setTargetPageUid($row['pid'])->setArguments(
					['tx_sgjobs_jobapplication' => ['jobId' => $job['uid']]]
				)->setCreateAbsoluteUri(TRUE)->buildFrontendUri();

				$pageList[] = [
					'url' => $url,
					'title' => ''
				];
			}
		}
	}

	/**
	 * Initializes TSFE and sets $GLOBALS['TSFE']
	 *
	 * @return    void
	 * @throws \RuntimeException
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Core\Error\Http\ServiceUnavailableException
	 */
	private function initTSFE() {
		$GLOBALS['TSFE'] = $tsfe = GeneralUtility::makeInstance(
			TypoScriptFrontendController::class, $GLOBALS['TYPO3_CONF_VARS'], GeneralUtility::_GP('id'), ''
		);
		/** @var \TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController $tsfe */
		$tsfe->connectToDB();
		$tsfe->initFEuser();
		EidUtility::initTCA();
		$tsfe->determineId();
		$tsfe->initTemplate();
		$tsfe->getConfigArray();
		$tsfe->settingLanguage();

		// Get linkVars, absRefPrefix, etc
		PageGenerator::pagegenInit();
	}

	/**
	 * get all jobs stored on specific page ids
	 *
	 * @param string $pid
	 * @return array|null
	 */
	private function getJobsByPid($pid) {
		/** @var $databaseConnection DatabaseConnection */
		$databaseConnection = $GLOBALS['TYPO3_DB'];
		$rows = $databaseConnection->exec_SELECTgetRows(
			'*',
			'tx_sgjobs_domain_model_job',
			'pid IN (' . $pid . ')'
		);

		return $rows;
	}
}
